@@ -64,6 +64,7 @@
 //! # }
 //! ```
 mod transports;
+use futures_util::future::join_all;
 pub use transports::*;
 
 mod provider;
@@ -74,6 +75,9 @@ pub mod ens;
 mod pending_transaction;
 pub use pending_transaction::PendingTransaction;
 
+mod pending_escalator;
+pub use pending_escalator::EscalatingPending;
+
 mod stream;
 pub use futures_util::StreamExt;
 pub use stream::{interval, FilterWatcher, TransactionStream, DEFAULT_POLL_INTERVAL};
@@ -89,6 +93,9 @@ use std::{error::Error, fmt::Debug, future::Future, pin::Pin, str::FromStr};
 
 pub use provider::{FilterKind, Provider, ProviderError};
 
+/// A simple gas escalation policy
+pub type EscalationPolicy = Box<dyn Fn(U256, usize) -> U256 + Send + Sync>;
+
 // Helper type alias
 #[cfg(target_arch = "wasm32")]
 pub(crate) type PinBoxFut<'a, T> = Pin<Box<dyn Future<Output = Result<T, ProviderError>> + 'a>>;
@@ -109,7 +116,7 @@ pub trait JsonRpcClient: Debug + Send + Sync {
     async fn request<T, R>(&self, method: &str, params: T) -> Result<R, Self::Error>
     where
         T: Debug + Serialize + Send + Sync,
-        R: Serialize + DeserializeOwned;
+        R: DeserializeOwned;
 }
 
 use ethers_core::types::*;
@@ -132,9 +139,9 @@ where
 /// A middleware allows customizing requests send and received from an ethereum node.
 ///
 /// Writing a middleware is as simple as:
-/// 1. implementing the [`inner`](crate::Middleware::inner) method to point to the next layer in the "middleware onion",
-/// 2. implementing the [`FromErr`](crate::FromErr) trait on your middleware's error type
-/// 3. implementing any of the methods you want to override
+/// 1. implementing the [`inner`](crate::Middleware::inner) method to point to the next layer in the
+/// "middleware onion", 2. implementing the [`FromErr`](crate::FromErr) trait on your middleware's
+/// error type 3. implementing any of the methods you want to override
 ///
 /// ```rust
 /// use ethers_providers::{Middleware, FromErr};
@@ -215,80 +222,58 @@ pub trait Middleware: Sync + Send + Debug {
         tx: &mut TypedTransaction,
         block: Option<BlockId>,
     ) -> Result<(), Self::Error> {
-        let mut tx_clone = tx.clone();
+        if let Some(default_sender) = self.default_sender() {
+            if tx.from().is_none() {
+                tx.set_from(default_sender);
+            }
+        }
+
+        // TODO: Can we poll the futures below at the same time?
+        // Access List + Name resolution and then Gas price + Gas
+
+        // set the ENS name
+        if let Some(NameOrAddress::Name(ref ens_name)) = tx.to() {
+            let addr = self.resolve_name(ens_name).await?;
+            tx.set_to(addr);
+        }
+
+        // estimate the gas without the access list
+        let gas = maybe(tx.gas().cloned(), self.estimate_gas(tx)).await?;
+        let mut al_used = false;
+
+        // set the access lists
+        if let Some(access_list) = tx.access_list() {
+            if access_list.0.is_empty() {
+                if let Ok(al_with_gas) = self.create_access_list(tx, block).await {
+                    // only set the access list if the used gas is less than the
+                    // normally estimated gas
+                    if al_with_gas.gas_used < gas {
+                        tx.set_access_list(al_with_gas.access_list);
+                        tx.set_gas(al_with_gas.gas_used);
+                        al_used = true;
+                    }
+                }
+            }
+        }
+
+        if !al_used {
+            tx.set_gas(gas);
+        }
 
-        // TODO: Maybe deduplicate the code in a nice way
         match tx {
-            TypedTransaction::Legacy(ref mut inner) => {
-                if let Some(NameOrAddress::Name(ref ens_name)) = inner.to {
-                    let addr = self.resolve_name(ens_name).await?;
-                    inner.to = Some(addr.into());
-                    tx_clone.set_to(addr);
-                };
-
-                if inner.from.is_none() {
-                    inner.from = self.default_sender();
-                }
-
-                let (gas_price, gas) = futures_util::try_join!(
-                    maybe(inner.gas_price, self.get_gas_price()),
-                    maybe(inner.gas, self.estimate_gas(&tx_clone)),
-                )?;
-                inner.gas = Some(gas);
-                inner.gas_price = Some(gas_price);
+            TypedTransaction::Eip2930(_) | TypedTransaction::Legacy(_) => {
+                let gas_price = maybe(tx.gas_price(), self.get_gas_price()).await?;
+                tx.set_gas_price(gas_price);
             }
-            TypedTransaction::Eip2930(inner) => {
-                if let Ok(lst) = self.create_access_list(&tx_clone, block).await {
-                    inner.access_list = lst.access_list;
-                }
-
-                if let Some(NameOrAddress::Name(ref ens_name)) = inner.tx.to {
-                    let addr = self.resolve_name(ens_name).await?;
-                    inner.tx.to = Some(addr.into());
-                    tx_clone.set_to(addr);
-                };
-
-                if inner.tx.from.is_none() {
-                    inner.tx.from = self.default_sender();
-                }
-
-                let (gas_price, gas) = futures_util::try_join!(
-                    maybe(inner.tx.gas_price, self.get_gas_price()),
-                    maybe(inner.tx.gas, self.estimate_gas(&tx_clone)),
-                )?;
-                inner.tx.gas = Some(gas);
-                inner.tx.gas_price = Some(gas_price);
-            }
-            TypedTransaction::Eip1559(inner) => {
-                if let Ok(lst) = self.create_access_list(&tx_clone, block).await {
-                    inner.access_list = lst.access_list;
-                }
-
-                if let Some(NameOrAddress::Name(ref ens_name)) = inner.to {
-                    let addr = self.resolve_name(ens_name).await?;
-                    inner.to = Some(addr.into());
-                    tx_clone.set_to(addr);
-                };
-
-                if inner.from.is_none() {
-                    inner.from = self.default_sender();
-                }
-
-                let gas = maybe(inner.gas, self.estimate_gas(&tx_clone)).await?;
-                inner.gas = Some(gas);
-
+            TypedTransaction::Eip1559(ref mut inner) => {
                 if inner.max_fee_per_gas.is_none() || inner.max_priority_fee_per_gas.is_none() {
                     let (max_fee_per_gas, max_priority_fee_per_gas) =
                         self.estimate_eip1559_fees(None).await?;
-                    if inner.max_fee_per_gas.is_none() {
-                        inner.max_fee_per_gas = Some(max_fee_per_gas);
-                    }
-                    if inner.max_priority_fee_per_gas.is_none() {
-                        inner.max_priority_fee_per_gas = Some(max_priority_fee_per_gas);
-                    }
-                }
+                    inner.max_fee_per_gas = Some(max_fee_per_gas);
+                    inner.max_priority_fee_per_gas = Some(max_priority_fee_per_gas);
+                };
             }
-        };
+        }
 
         Ok(())
     }
@@ -302,54 +287,75 @@ pub trait Middleware: Sync + Send + Debug {
         tx: T,
         block: Option<BlockId>,
     ) -> Result<PendingTransaction<'_, Self::Provider>, Self::Error> {
-        self.inner()
-            .send_transaction(tx, block)
-            .await
-            .map_err(FromErr::from)
+        self.inner().send_transaction(tx, block).await.map_err(FromErr::from)
+    }
+
+    /// Send a transaction with a simple escalation policy.
+    ///
+    /// `policy` should be a boxed function that maps `original_gas_price`
+    /// and `number_of_previous_escalations` -> `new_gas_price`.
+    ///
+    /// e.g. `Box::new(|start, escalation_index| start * 1250.pow(escalations) /
+    /// 1000.pow(escalations))`
+    async fn send_escalating<'a>(
+        &'a self,
+        tx: &TypedTransaction,
+        escalations: usize,
+        policy: EscalationPolicy,
+    ) -> Result<EscalatingPending<'a, Self::Provider>, Self::Error> {
+        let mut original = tx.clone();
+        self.fill_transaction(&mut original, None).await?;
+        let gas_price = original.gas_price().expect("filled");
+        let chain_id = self.get_chainid().await?.low_u64();
+        let sign_futs: Vec<_> = (0..escalations)
+            .map(|i| {
+                let new_price = policy(gas_price, i);
+                let mut r = original.clone();
+                r.set_gas_price(new_price);
+                r
+            })
+            .map(|req| async move {
+                self.sign_transaction(&req, self.default_sender().unwrap_or_default())
+                    .await
+                    .map(|sig| req.rlp_signed(chain_id, &sig))
+            })
+            .collect();
+
+        // we reverse for convenience. Ensuring that we can always just
+        // `pop()` the next tx off the back later
+        let mut signed = join_all(sign_futs).await.into_iter().collect::<Result<Vec<_>, _>>()?;
+        signed.reverse();
+
+        Ok(EscalatingPending::new(self.provider(), signed))
     }
 
     async fn resolve_name(&self, ens_name: &str) -> Result<Address, Self::Error> {
-        self.inner()
-            .resolve_name(ens_name)
-            .await
-            .map_err(FromErr::from)
+        self.inner().resolve_name(ens_name).await.map_err(FromErr::from)
     }
 
     async fn lookup_address(&self, address: Address) -> Result<String, Self::Error> {
-        self.inner()
-            .lookup_address(address)
-            .await
-            .map_err(FromErr::from)
+        self.inner().lookup_address(address).await.map_err(FromErr::from)
     }
 
     async fn get_block<T: Into<BlockId> + Send + Sync>(
         &self,
         block_hash_or_number: T,
     ) -> Result<Option<Block<TxHash>>, Self::Error> {
-        self.inner()
-            .get_block(block_hash_or_number)
-            .await
-            .map_err(FromErr::from)
+        self.inner().get_block(block_hash_or_number).await.map_err(FromErr::from)
     }
 
     async fn get_block_with_txs<T: Into<BlockId> + Send + Sync>(
         &self,
         block_hash_or_number: T,
     ) -> Result<Option<Block<Transaction>>, Self::Error> {
-        self.inner()
-            .get_block_with_txs(block_hash_or_number)
-            .await
-            .map_err(FromErr::from)
+        self.inner().get_block_with_txs(block_hash_or_number).await.map_err(FromErr::from)
     }
 
     async fn get_uncle_count<T: Into<BlockId> + Send + Sync>(
         &self,
         block_hash_or_number: T,
     ) -> Result<U256, Self::Error> {
-        self.inner()
-            .get_uncle_count(block_hash_or_number)
-            .await
-            .map_err(FromErr::from)
+        self.inner().get_uncle_count(block_hash_or_number).await.map_err(FromErr::from)
     }
 
     async fn get_uncle<T: Into<BlockId> + Send + Sync>(
@@ -357,10 +363,7 @@ pub trait Middleware: Sync + Send + Debug {
         block_hash_or_number: T,
         idx: U64,
     ) -> Result<Option<Block<H256>>, Self::Error> {
-        self.inner()
-            .get_uncle(block_hash_or_number, idx)
-            .await
-            .map_err(FromErr::from)
+        self.inner().get_uncle(block_hash_or_number, idx).await.map_err(FromErr::from)
     }
 
     async fn get_transaction_count<T: Into<NameOrAddress> + Send + Sync>(
@@ -368,10 +371,7 @@ pub trait Middleware: Sync + Send + Debug {
         from: T,
         block: Option<BlockId>,
     ) -> Result<U256, Self::Error> {
-        self.inner()
-            .get_transaction_count(from, block)
-            .await
-            .map_err(FromErr::from)
+        self.inner().get_transaction_count(from, block).await.map_err(FromErr::from)
     }
 
     async fn estimate_gas(&self, tx: &TypedTransaction) -> Result<U256, Self::Error> {
@@ -390,45 +390,37 @@ pub trait Middleware: Sync + Send + Debug {
         self.inner().get_chainid().await.map_err(FromErr::from)
     }
 
+    async fn get_net_version(&self) -> Result<U64, Self::Error> {
+        self.inner().get_net_version().await.map_err(FromErr::from)
+    }
+
     async fn get_balance<T: Into<NameOrAddress> + Send + Sync>(
         &self,
         from: T,
         block: Option<BlockId>,
     ) -> Result<U256, Self::Error> {
-        self.inner()
-            .get_balance(from, block)
-            .await
-            .map_err(FromErr::from)
+        self.inner().get_balance(from, block).await.map_err(FromErr::from)
     }
 
     async fn get_transaction<T: Send + Sync + Into<TxHash>>(
         &self,
         transaction_hash: T,
     ) -> Result<Option<Transaction>, Self::Error> {
-        self.inner()
-            .get_transaction(transaction_hash)
-            .await
-            .map_err(FromErr::from)
+        self.inner().get_transaction(transaction_hash).await.map_err(FromErr::from)
     }
 
     async fn get_transaction_receipt<T: Send + Sync + Into<TxHash>>(
         &self,
         transaction_hash: T,
     ) -> Result<Option<TransactionReceipt>, Self::Error> {
-        self.inner()
-            .get_transaction_receipt(transaction_hash)
-            .await
-            .map_err(FromErr::from)
+        self.inner().get_transaction_receipt(transaction_hash).await.map_err(FromErr::from)
     }
 
     async fn get_block_receipts<T: Into<BlockNumber> + Send + Sync>(
         &self,
         block: T,
     ) -> Result<Vec<TransactionReceipt>, Self::Error> {
-        self.inner()
-            .get_block_receipts(block)
-            .await
-            .map_err(FromErr::from)
+        self.inner().get_block_receipts(block).await.map_err(FromErr::from)
     }
 
     async fn get_gas_price(&self) -> Result<U256, Self::Error> {
@@ -439,10 +431,7 @@ pub trait Middleware: Sync + Send + Debug {
         &self,
         estimator: Option<fn(U256, Vec<Vec<U256>>) -> (U256, U256)>,
     ) -> Result<(U256, U256), Self::Error> {
-        self.inner()
-            .estimate_eip1559_fees(estimator)
-            .await
-            .map_err(FromErr::from)
+        self.inner().estimate_eip1559_fees(estimator).await.map_err(FromErr::from)
     }
 
     async fn get_accounts(&self) -> Result<Vec<Address>, Self::Error> {
@@ -453,10 +442,7 @@ pub trait Middleware: Sync + Send + Debug {
         &'a self,
         tx: Bytes,
     ) -> Result<PendingTransaction<'a, Self::Provider>, Self::Error> {
-        self.inner()
-            .send_raw_transaction(tx)
-            .await
-            .map_err(FromErr::from)
+        self.inner().send_raw_transaction(tx).await.map_err(FromErr::from)
     }
 
     /// This returns true if either the middleware stack contains a `SignerMiddleware`, or the
@@ -474,6 +460,15 @@ pub trait Middleware: Sync + Send + Debug {
         self.inner().sign(data, from).await.map_err(FromErr::from)
     }
 
+    /// Sign a transaction via RPC call
+    async fn sign_transaction(
+        &self,
+        tx: &TypedTransaction,
+        from: Address,
+    ) -> Result<Signature, Self::Error> {
+        self.inner().sign_transaction(tx, from).await.map_err(FromErr::from)
+    }
+
     ////// Contract state
 
     async fn get_logs(&self, filter: &Filter) -> Result<Vec<Log>, Self::Error> {
@@ -488,10 +483,7 @@ pub trait Middleware: Sync + Send + Debug {
         &self,
         id: T,
     ) -> Result<bool, Self::Error> {
-        self.inner()
-            .uninstall_filter(id)
-            .await
-            .map_err(FromErr::from)
+        self.inner().uninstall_filter(id).await.map_err(FromErr::from)
     }
 
     async fn watch<'a>(
@@ -504,10 +496,7 @@ pub trait Middleware: Sync + Send + Debug {
     async fn watch_pending_transactions(
         &self,
     ) -> Result<FilterWatcher<'_, Self::Provider, H256>, Self::Error> {
-        self.inner()
-            .watch_pending_transactions()
-            .await
-            .map_err(FromErr::from)
+        self.inner().watch_pending_transactions().await.map_err(FromErr::from)
     }
 
     async fn get_filter_changes<T, R>(&self, id: T) -> Result<Vec<R>, Self::Error>
@@ -515,10 +504,7 @@ pub trait Middleware: Sync + Send + Debug {
         T: Into<U256> + Send + Sync,
         R: Serialize + DeserializeOwned + Send + Sync + Debug,
     {
-        self.inner()
-            .get_filter_changes(id)
-            .await
-            .map_err(FromErr::from)
+        self.inner().get_filter_changes(id).await.map_err(FromErr::from)
     }
 
     async fn watch_blocks(&self) -> Result<FilterWatcher<'_, Self::Provider, H256>, Self::Error> {
@@ -530,10 +516,7 @@ pub trait Middleware: Sync + Send + Debug {
         at: T,
         block: Option<BlockId>,
     ) -> Result<Bytes, Self::Error> {
-        self.inner()
-            .get_code(at, block)
-            .await
-            .map_err(FromErr::from)
+        self.inner().get_code(at, block).await.map_err(FromErr::from)
     }
 
     async fn get_storage_at<T: Into<NameOrAddress> + Send + Sync>(
@@ -542,10 +525,7 @@ pub trait Middleware: Sync + Send + Debug {
         location: H256,
         block: Option<BlockId>,
     ) -> Result<H256, Self::Error> {
-        self.inner()
-            .get_storage_at(from, location, block)
-            .await
-            .map_err(FromErr::from)
+        self.inner().get_storage_at(from, location, block).await.map_err(FromErr::from)
     }
 
     async fn get_proof<T: Into<NameOrAddress> + Send + Sync>(
@@ -554,10 +534,7 @@ pub trait Middleware: Sync + Send + Debug {
         locations: Vec<H256>,
         block: Option<BlockId>,
     ) -> Result<EIP1186ProofResponse, Self::Error> {
-        self.inner()
-            .get_proof(from, locations, block)
-            .await
-            .map_err(FromErr::from)
+        self.inner().get_proof(from, locations, block).await.map_err(FromErr::from)
     }
 
     // Mempool inspection for Geth's API
@@ -583,10 +560,7 @@ pub trait Middleware: Sync + Send + Debug {
         trace_type: Vec<TraceType>,
         block: Option<BlockNumber>,
     ) -> Result<BlockTrace, Self::Error> {
-        self.inner()
-            .trace_call(req, trace_type, block)
-            .await
-            .map_err(FromErr::from)
+        self.inner().trace_call(req, trace_type, block).await.map_err(FromErr::from)
     }
 
     /// Traces a call to `eth_sendRawTransaction` without making the call, returning the traces
@@ -595,10 +569,7 @@ pub trait Middleware: Sync + Send + Debug {
         data: Bytes,
         trace_type: Vec<TraceType>,
     ) -> Result<BlockTrace, Self::Error> {
-        self.inner()
-            .trace_raw_transaction(data, trace_type)
-            .await
-            .map_err(FromErr::from)
+        self.inner().trace_raw_transaction(data, trace_type).await.map_err(FromErr::from)
     }
 
     /// Replays a transaction, returning the traces
@@ -607,10 +578,7 @@ pub trait Middleware: Sync + Send + Debug {
         hash: H256,
         trace_type: Vec<TraceType>,
     ) -> Result<BlockTrace, Self::Error> {
-        self.inner()
-            .trace_replay_transaction(hash, trace_type)
-            .await
-            .map_err(FromErr::from)
+        self.inner().trace_replay_transaction(hash, trace_type).await.map_err(FromErr::from)
     }
 
     /// Replays all transactions in a block returning the requested traces for each transaction
@@ -619,10 +587,7 @@ pub trait Middleware: Sync + Send + Debug {
         block: BlockNumber,
         trace_type: Vec<TraceType>,
     ) -> Result<Vec<BlockTrace>, Self::Error> {
-        self.inner()
-            .trace_replay_block_transactions(block, trace_type)
-            .await
-            .map_err(FromErr::from)
+        self.inner().trace_replay_block_transactions(block, trace_type).await.map_err(FromErr::from)
     }
 
     /// Returns traces created at given block
@@ -632,10 +597,7 @@ pub trait Middleware: Sync + Send + Debug {
 
     /// Return traces matching the given filter
     async fn trace_filter(&self, filter: TraceFilter) -> Result<Vec<Trace>, Self::Error> {
-        self.inner()
-            .trace_filter(filter)
-            .await
-            .map_err(FromErr::from)
+        self.inner().trace_filter(filter).await.map_err(FromErr::from)
     }
 
     /// Returns trace at the given position
@@ -644,18 +606,12 @@ pub trait Middleware: Sync + Send + Debug {
         hash: H256,
         index: Vec<T>,
     ) -> Result<Trace, Self::Error> {
-        self.inner()
-            .trace_get(hash, index)
-            .await
-            .map_err(FromErr::from)
+        self.inner().trace_get(hash, index).await.map_err(FromErr::from)
     }
 
     /// Returns all traces of a given transaction
     async fn trace_transaction(&self, hash: H256) -> Result<Vec<Trace>, Self::Error> {
-        self.inner()
-            .trace_transaction(hash)
-            .await
-            .map_err(FromErr::from)
+        self.inner().trace_transaction(hash).await.map_err(FromErr::from)
     }
 
     // Parity namespace
@@ -665,10 +621,7 @@ pub trait Middleware: Sync + Send + Debug {
         &self,
         block: T,
     ) -> Result<Vec<TransactionReceipt>, Self::Error> {
-        self.inner()
-            .parity_block_receipts(block)
-            .await
-            .map_err(FromErr::from)
+        self.inner().parity_block_receipts(block).await.map_err(FromErr::from)
     }
 
     async fn subscribe<T, R>(
@@ -706,10 +659,7 @@ pub trait Middleware: Sync + Send + Debug {
     where
         <Self as Middleware>::Provider: PubsubClient,
     {
-        self.inner()
-            .subscribe_pending_txs()
-            .await
-            .map_err(FromErr::from)
+        self.inner().subscribe_pending_txs().await.map_err(FromErr::from)
     }
 
     async fn subscribe_logs<'a>(
@@ -719,10 +669,7 @@ pub trait Middleware: Sync + Send + Debug {
     where
         <Self as Middleware>::Provider: PubsubClient,
     {
-        self.inner()
-            .subscribe_logs(filter)
-            .await
-            .map_err(FromErr::from)
+        self.inner().subscribe_logs(filter).await.map_err(FromErr::from)
     }
 
     async fn fee_history<T: Into<U256> + serde::Serialize + Send + Sync>(
@@ -742,10 +689,7 @@ pub trait Middleware: Sync + Send + Debug {
         tx: &TypedTransaction,
         block: Option<BlockId>,
     ) -> Result<AccessListWithGasUsed, Self::Error> {
-        self.inner()
-            .create_access_list(tx, block)
-            .await
-            .map_err(FromErr::from)
+        self.inner().create_access_list(tx, block).await.map_err(FromErr::from)
     }
 }
 
@@ -787,9 +731,6 @@ pub trait CeloMiddleware: Middleware {
         &self,
         block_id: T,
     ) -> Result<Vec<String>, ProviderError> {
-        self.provider()
-            .get_validators_bls_public_keys(block_id)
-            .await
-            .map_err(FromErr::from)
+        self.provider().get_validators_bls_public_keys(block_id).await.map_err(FromErr::from)
     }
 }
